/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.figure;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Shape;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.fortiss.tooling.base.ui.ToolingBaseUIActivator;

/**
 * This shape consists of a rounded rectangle that is surrounded by transparent
 * insets. The rounded rectangle is filled with {@link #getBackgroundColor()}. A
 * light top-down gradient from white to the background color is painted over
 * the background color to achieve a 3 dimensional reflection effect.
 * <p>
 * Anti-aliasing is enabled for smooth corners of the rounded rectangle.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: C452794B56B9A854A801D112102E7571
 */
public class PrettyRoundedRectangle extends Shape {

	/** The radius to use for the rounded corners of this shape. */
	protected final int cornerRadius;

	/** The transparent insets surrounding this shape. */
	protected final Insets insets;

	/** A linear blend applied at the top. */
	protected static final Image linearBlend = ToolingBaseUIActivator
			.getImageDescriptor("icons/overlay/linear-blend-round.png")
			.createImage();

	/** A radial blend applied at the bottom. */
	protected static final Image radialBlend = ToolingBaseUIActivator
			.getImageDescriptor("icons/overlay/radial-blend.png").createImage();

	/** Constructor. */
	public PrettyRoundedRectangle(int cornerRadius, Insets insets) {
		this.cornerRadius = cornerRadius;
		this.insets = insets;
	}

	/** {@inheritDoc} */
	@Override
	protected void fillShape(Graphics gfx) {
		EVisualStyle style = EVisualStyle.get();
		gfx.pushState();
		if (style == EVisualStyle.Glossy) {
			// Glossy style.
			final Rectangle b = getCroppedBounds();
			gfx.setBackgroundColor(ColorConstants.black);
			gfx.setAlpha(48);
			gfx.fillRoundRectangle(b.getCopy().translate(5, 5), cornerRadius,
					cornerRadius);
			gfx.restoreState();
			gfx.fillRoundRectangle(b, cornerRadius, cornerRadius);
			gfx.setAlpha(255);
			gfx.drawImage(linearBlend, new Rectangle(linearBlend.getBounds()),
					b.getCopy().resize(-1, -b.height / 2 - 1));
			gfx.setAlpha(200);
			gfx.drawImage(
					radialBlend,
					new Rectangle(radialBlend.getBounds()),
					b.getCopy().resize(-1, -b.height / 2 - 1)
							.translate(0, b.height / 2));
		} else {
			// Traditional style is default.
			gfx.fillRoundRectangle(getCroppedBounds(), cornerRadius,
					cornerRadius);
			gfx.setForegroundColor(ColorConstants.white);
			gfx.setAlpha(200);
			gfx.fillGradient(getCroppedBounds().resize(0, -cornerRadius), true);
		}
		gfx.popState();
	}

	/** Returns a copy of the bounds cropped by the insets. */
	protected Rectangle getCroppedBounds() {
		return bounds.getShrinked(insets);
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Additionally enables anti-aliasing.
	 */
	@Override
	protected void outlineShape(Graphics gfx) {
		final EVisualStyle style = EVisualStyle.get();
		gfx.pushState();
		gfx.setAntialias(SWT.ON);
		if (style == EVisualStyle.Glossy) {
			// Glossy style.
			gfx.setLineCap(SWT.CAP_ROUND);
			int scale = (int) gfx.getAbsoluteScale();
			gfx.setLineWidth(scale + 1);
			Color bg = gfx.getBackgroundColor();
			Color bgdark = new Color(bg.getDevice(), Math.max(0,
					(bg.getRed() - 80)), Math.max(0, (bg.getGreen() - 80)),
					Math.max(0, (bg.getBlue() - 80)));
			gfx.setForegroundColor(bgdark);
			gfx.drawRoundRectangle(getCroppedBounds().resize(-1, -1),
					cornerRadius, cornerRadius);
			bgdark.dispose();
		} else {
			// Traditional style is default.
			gfx.drawRoundRectangle(getCroppedBounds().resize(-1, -1),
					cornerRadius, cornerRadius);
		}
		gfx.popState();
	}
}
