/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.utils;

import static org.eclipse.draw2d.PositionConstants.EAST;
import static org.eclipse.draw2d.PositionConstants.SOUTH;
import static org.eclipse.draw2d.PositionConstants.SOUTH_EAST;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_ASPECT_RATIO;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_HEIGHT;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_WIDTH;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.getConnectorAngleAsDouble;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.getNodePosition;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.getNodeSize;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.setConnectorAngle;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;

/**
 * Utility methods for ellipse-shaped edit part layout.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 9BDB7D27FC9B1917E41C3DFD44C0FF23
 */
public final class EllipseLayoutUtils {
	/**
	 * Returns polar coordinates of the point on the ellipse border having the
	 * given angle.
	 * 
	 * @param angle
	 *            the angle
	 * @param ellipseSize
	 *            the wdith and height of the ellipse
	 * @param inset
	 *            the insets of the ellipse figure
	 * @return the polar coordinates
	 */
	public static Polar calculateEllipseBorderPolar(double angle,
			Dimension ellipseSize, int inset) {
		double a = ellipseSize.width / 2 - inset;
		double b = ellipseSize.height / 2 - inset;
		double epsilon = Math.sqrt(a * a - b * b) / a;
		double r = b
				/ Math.sqrt(1 - epsilon * epsilon * Math.cos(angle)
						* Math.cos(angle));
		return new Polar(angle, r);
	}

	/**
	 * Returns Cartesian coordinates of the point on the ellipse border having
	 * the given angle.
	 * 
	 * @param angle
	 *            the angle
	 * @param ellipseSize
	 *            the wdith and height of the ellipse
	 * @param inset
	 *            the insets of the ellipse figure
	 * @return the point coordinates
	 */
	public static Point calculateEllipseBorderPoint(double angle,
			Dimension ellipseSize, int inset) {
		Polar polar = calculateEllipseBorderPolar(angle, ellipseSize, inset);
		return polar.toCartesian().translate(ellipseSize.getScaled(.5));
	}

	/**
	 * Returns a radian value for a given degree value
	 * 
	 * @param degree
	 *            the degree (0 to 360)
	 * @return rad (0 to 2*PI)
	 */
	public static double getRadianFromDegree(double degree) {
		return (Math.PI / 180.0) * degree;
	}

	/**
	 * Layouts the given connector within the surrounding ellipse.
	 * 
	 * @param layouted
	 *            the layouted connector model element
	 * @param container
	 *            the container ellipse
	 * @param location
	 *            the target point to be moved to the ellipse border
	 */
	public static void layoutConnectorInEllipse(ILayoutedModelElement layouted,
			ILayoutedModelElement container, Point location) {
		Point position = location.getCopy();
		org.fortiss.tooling.base.model.layout.Point np = getNodePosition(container);
		position.translate(-np.getX(), -np.getY());

		org.fortiss.tooling.base.model.layout.Dimension nd = getNodeSize(container);
		Dimension d = new Dimension(nd.getWidth(), nd.getHeight());
		position.translate(d.scale(-.5));

		// store the value in the model
		setConnectorAngle(layouted, new Polar(position).angle);
	}

	/**
	 * Ensure that the minimum size and aspect ratio of a oval shape are
	 * redeemed.
	 * 
	 * @param oldSize
	 *            the old ellipse size
	 * @param direction
	 *            the resize direction
	 * @return the clamped shape size
	 */
	public static Dimension clampEllipseShapeSize(Dimension oldSize,
			int direction) {
		return clampEllipseShapeSize(oldSize, direction,
				DEFAULT_SHAPE_MINIMUM_WIDTH, DEFAULT_SHAPE_MINIMUM_HEIGHT,
				DEFAULT_SHAPE_ASPECT_RATIO);
	}

	/**
	 * Ensure that the minimum size and aspect ratio of a oval shape are
	 * redeemed.
	 * 
	 * @param oldSize
	 *            the old ellipse size
	 * @param direction
	 *            the resize direction
	 * @param minimumWidth
	 *            the minimal width
	 * @param minimumHeight
	 *            the minimal height
	 * @param aspectRatio
	 *            the aspect ratio
	 * @return the clamped shape size
	 */
	public static Dimension clampEllipseShapeSize(Dimension oldSize,
			int direction, int minimumWidth, int minimumHeight,
			double aspectRatio) {
		Dimension newSize;

		// check for minimum size
		newSize = new Dimension(oldSize.width < minimumWidth ? minimumWidth
				: oldSize.width, oldSize.height < minimumHeight ? minimumHeight
				: oldSize.height);

		// ensure that the aspect ratio is redeemed
		switch (direction) {
		case SOUTH_EAST:
			int aspectHeight = (int) (newSize.width / aspectRatio);
			int aspectWidth = (int) (newSize.height * aspectRatio);
			newSize.setSize(new Dimension(
					newSize.width > aspectWidth ? aspectWidth : newSize.width,
					newSize.height > aspectHeight ? aspectHeight
							: newSize.height));
			break;
		case SOUTH:
			newSize.setSize(new Dimension((int) (newSize.height * aspectRatio),
					newSize.height));
			break;
		case EAST:
			newSize.setSize(new Dimension(newSize.width,
					(int) (newSize.width / aspectRatio)));
			break;
		}

		return newSize;
	}

	/**
	 * Returns the bounds rectangle of the connector given its parent bounds.
	 * 
	 * @param connector
	 *            the connector element
	 * @param parent
	 *            the parent element
	 * @return the connector bounds
	 */
	public static Rectangle getAbsoluteConnectorBounds(
			ILayoutedModelElement connector, ILayoutedModelElement parent) {

		Rectangle result = new Rectangle(0, 0, DEFAULT_CONNECTOR_SIZE,
				DEFAULT_CONNECTOR_SIZE);
		double angle = getConnectorAngleAsDouble(connector);
		org.fortiss.tooling.base.model.layout.Dimension parentSize = getNodeSize(parent);

		Dimension parentDim = new Dimension(parentSize.getWidth(),
				parentSize.getHeight());
		Point position = calculateEllipseBorderPoint(angle, parentDim,
				DEFAULT_CONNECTOR_SIZE / 2).translate(
				-DEFAULT_CONNECTOR_SIZE / 2, -DEFAULT_CONNECTOR_SIZE / 2);
		org.fortiss.tooling.base.model.layout.Point parentPos = getNodePosition(parent);
		result.x = position.x + parentPos.getX();
		result.y = position.y + parentPos.getY();

		return result;
	}

	/**
	 * Class for storing polar coordinates.
	 * 
	 * The angles are interpreted clockwise in the following way: 0: East;
	 * 90(pi/2): South; 180(pi): West; 270(3/2pi): North
	 * 
	 * @author wandinger
	 * @author hummel
	 * @author $Author$
	 * @version $Rev$
	 * @levd.rating YELLOW Rev: 519
	 */
	public static class Polar {

		/** The angle */
		public double angle;

		/** The distance */
		public double distance;

		/** The constructor */
		public Polar(double angle, double distance) {
			this.angle = angle;
			this.distance = distance;
		}

		/** Create new polar values from cartesian ones. */
		public Polar(Point pos) {
			angle = Math.atan2(pos.y, pos.x);
			distance = Math.sqrt(pos.x * pos.x + pos.y * pos.y);
		}

		/** Returns a cartesian representation. */
		public Point toCartesian() {
			int x = (int) (distance * Math.cos(angle));
			int y = (int) (distance * Math.sin(angle));
			return new Point(x, y);
		}
	}
}
