/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation;
import org.fortiss.tooling.kernel.service.IConstraintCheckerService;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;

/**
 * This class implements the {@link IConstraintCheckerService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: B377B07759BBBCC86D213E0ECE3CC509
 */
public class ConstraintCheckerService extends
		EObjectAwareServiceBase<IConstraintChecker<EObject>> implements
		IConstraintCheckerService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.modelElementConstraintChecker";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelElementConstraintChecker";

	/** The compositor class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "checker";

	/** {@inheritDoc} */
	@Override
	public List<IConstraintViolation<? extends EObject>> performAllConstraintChecksRecursively(
			EObject modelElement) {
		List<IConstraintViolation<? extends EObject>> result = new LinkedList<IConstraintViolation<? extends EObject>>();
		performConstraintCheck(modelElement, result);
		for (Iterator<EObject> iter = modelElement.eAllContents(); iter
				.hasNext();) {
			performConstraintCheck(iter.next(), result);
		}
		Collections.sort(result, IConstraintViolation.SEVERITY_COMPARATOR);
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public List<IConstraintViolation<? extends EObject>> performAllConstraintChecks(
			EObject modelElement) {
		List<IConstraintViolation<? extends EObject>> result = new LinkedList<IConstraintViolation<? extends EObject>>();
		performConstraintCheck(modelElement, result);
		Collections.sort(result, IConstraintViolation.SEVERITY_COMPARATOR);
		return result;
	}

	/**
	 * Performs all constraint checks on the given model element. Violations are
	 * added to the given list. Constraint checks on content elements are not
	 * considered.
	 */
	private void performConstraintCheck(EObject modelElement,
			List<IConstraintViolation<? extends EObject>> violationList) {
		List<IConstraintChecker<EObject>> handlers = getRegisteredHandlers(modelElement
				.getClass());
		if (handlers == null) {
			return;
		}
		for (IConstraintChecker<EObject> checker : handlers) {
			if (checker.isApplicable(modelElement)) {
				List<? extends IConstraintViolation<? extends EObject>> violation = checker
						.apply(modelElement);
				if (violation != null) {
					violationList.addAll(violation);
				}
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<IConstraintChecker<? extends EObject>> getAllConstraintCheckers(
			EObject modelElement) {
		return new ArrayList<IConstraintChecker<? extends EObject>>(
				getRegisteredHandlers(modelElement.getClass()));
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}
}
