/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.Iterator;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.model.IIdLabeledReference;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Utility methods for accessing {@link IProjectRootElement}s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: BFAEE573B73BEA35ED5EC160F4015FB5
 */
public final class KernelModelElementUtils {

	/**
	 * Returns the instance of a project root element corresponding to the given
	 * class.
	 * 
	 * @param element
	 *            a model element contained in the project to be searched for
	 *            the root element.
	 * @param clazz
	 *            the class of the root element to be returned.
	 * @return instance of class <code>clazz</code> or <code>null</code> if no
	 *         such root element was found.
	 */
	public static <T extends IProjectRootElement> T getRootElement(
			EObject element, Class<T> clazz) {
		return EcoreUtils.pickFirstInstanceOf(clazz,
				IPersistencyService.INSTANCE.getTopLevelElementFor(element)
						.getRootModelElement().eContents());
	}

	/**
	 * Finds the model element referenced by the given
	 * {@link IIdLabeledReference} element or <code>null</code> if no such
	 * element exists. The reference's model project is searched from its root.
	 * 
	 * @param reference
	 *            the ID-reference to be searched
	 * @return the referenced element or <code>null</code>.
	 */
	public static EObject findReferencedElement(IIdLabeledReference reference) {
		return findElementById(reference.getIdReference(),
				IPersistencyService.INSTANCE.getTopLevelElementFor(reference)
						.getRootModelElement());

	}

	/**
	 * Finds the model element referenced by the given
	 * {@link IIdLabeledReference} element or <code>null</code> if no such
	 * element exists.
	 * 
	 * @param id
	 *            the ID of the element to be searched.
	 * @param root
	 *            the root model element to be searched.
	 * @return the referenced element or <code>null</code>.
	 */
	public static EObject findElementById(int id, EObject root) {
		for (Iterator<EObject> iter = root.eAllContents(); iter.hasNext();) {
			EObject eo = iter.next();
			if (eo instanceof IIdLabeled && ((IIdLabeled) eo).getId() == id) {
				return eo;
			}
		}
		return null;
	}

	/**
	 * Determines if the given element is an indirect child of the given parent
	 * element.
	 * 
	 * @param element
	 *            the child model element
	 * @param parentElement
	 *            the parent model element
	 * @return <code>true</code> if the element is a child in the aggregation
	 *         tree of the parent element.
	 */
	public static boolean isChildElementOf(EObject element,
			EObject parentElement) {
		while (element.eContainer() != null) {
			if (element.eContainer() == parentElement) {
				return true;
			}
			element = element.eContainer();
		}
		return false;
	}
}
